unit MainForm;

interface

uses
  System.SysUtils, System.Types, System.UITypes, System.Classes, System.Variants,
  FMX.Types, FMX.Controls, FMX.Forms, FMX.Graphics, FMX.Dialogs, FMX.Layouts,
  FMX.Memo, FMX.StdCtrls, FMX.Edit,
  FMX.Platform,
  TypInfo,
  Androidapi.JNIBridge,
  Androidapi.JNI.JavaTypes,
  Androidapi.Jni,
  Androidapi.JNI.Dalvik;

type
  TForm2 = class(TForm)
    ToolBar1: TToolBar;
    Edit1: TEdit;
    Button1: TButton;
    Button2: TButton;
    Memo1: TMemo;
    procedure FormCreate(Sender: TObject);
    procedure Button1Click(Sender: TObject);
  private
    { Private declarations }
    procedure DoSetups;
    function CheckForCustomJavaJarFile : Boolean;
    procedure LoadCustomJavaClass;

  public
    data:string;
    JavaEnv:PJNIEnv;
    JavaObject:JObject;
    JavaObjectID:JNIObject;
    GetMethod:JNIMethodID;
    SetMethod:JNIMethodID;
    StuntMethod:JNIMethodID;
    StrData: JNIString;

    { Public declarations }
    constructor Create(AOwner : TComponent); override;
    destructor Destroy; override;
  end;

var
  Form2: TForm2;

implementation

{$R *.fmx}

uses FMX.Helpers.Android, Androidapi.JNI.GraphicsContentViewText, IOUtils;

const CUSTOMJARFILE = 'MyXE5Tests-debug-unaligned.apk';
const javaClassName= 'com/gli/myxe5tests/RTClassWrapTest1';

procedure TForm2.Button1Click(Sender: TObject);
var
  CL:JDexClassLoader;
  jTempClass:Jlang_Class;
  jTemp:JObject;
  oTemp:TObject;
  jLocalInterface:ILocalObject;
  context:JContext;
  optimizedpath_jfile:JFile;
  dexpath_jstring,optimizedpath_jstring:JString;
  ji:JNIInt;
  jv:JNIValue;
  x:array of JNIOBJECT;
begin
  context:=SharedActivityContext;
  JavaEnv:=TJNIResolver.GetJNIEnv;


  Memo1.lines.Add('Loading external library from "'+CUSTOMJARFILE+'"');
  dexpath_jstring := StringToJString(TPath.GetDocumentsPath + PathDelim + CUSTOMJARFILE);

  // locate/create a directory where our dex files can be put
  optimizedpath_jfile := context.getDir(StringToJString('outdex'),TJContext.javaclass.mode_private);


  optimizedpath_jstring := optimizedpath_jfile.getAbsolutePath;

  Memo1.lines.Add('Path for DEX files = ' + JStringToString(optimizedpath_jstring));
  Memo1.lines.Add('APK containing target class = ' + JStringToString(dexpath_jstring));

  cl := TJDexClassLoader.JavaClass.init
    (dexpath_jstring,optimizedpath_jstring,nil,TJDexClassLoader.JavaClass.getSystemClassLoader);

  if not assigned(cl) then
  begin
    Memo1.Lines.Add('?Failed to get DEXClassLoader');
    exit;
  end
  else
    Memo1.Lines.Add('DEXClassLoader successed!');



  jTempClass := cl.loadClass(StringToJString(javaClassName));

  if assigned(jTempClass) then
  begin
    jTemp := jTempClass;    // N.B You could now import the entire class


    if (jTemp.QueryInterface(ILocalObject,jLocalInterface) = S_OK) then
    begin
      // supports ilocalobject
      JavaObject := jTempClass.newInstance;
      oTemp := JavaObject as TObject;
      JavaObjectID := tjavaimport(otemp).GetObjectID;
      Memo1.Lines.Add(oTemp.ClassName);

      GetMethod := TJNIResolver.GetJavaMethodID((jTempClass as ILocalObject).GetObjectID,'getIntValue','()I');
      if not assigned(Getmethod) then
      begin
        memo1.Lines.Add('?getIntvalue not supported');
        exit;
      end;

      ji:=MaxInt;
      ji:=JavaEnv^.CallIntMethod(JavaEnv,JavaObjectID,GetMethod);
      memo1.Lines.Add('getIntValue returns '+inttostr(ji));

      SetMethod:=TJNIResolver.GetJavaMethodID((jTempClass as ILocalObject).GetObjectID,'setIntValue','(I)V');
      if not assigned(Setmethod) then
      begin
        memo1.Lines.Add('?setIntvalue not supported');
        exit;
      end;

      ji:=1234;
      jv.i:=ji;
      JavaEnv^.CallVoidMethodA(JavaEnv,JavaObjectID,SetMethod,@jv);

      ji:=JavaEnv^.CallIntMethod(JavaEnv,JavaObjectID,GetMethod);
      memo1.Lines.Add('External method returns '+inttostr(ji));

      StuntMethod:=TJNIResolver.GetJavaMethodID(
        (jTempClass as ILocalObject).GetObjectID,'stunt','(Ljava/lang/String;I)V');
      if not assigned(stuntMethod) then
      begin
        memo1.Lines.Add('?stunt not supported');
        exit;
      end;
      // there are two parameters
      setlength(x,2);

      // first parameter is a string
      x[0]:=StringToJNIString(JavaEnv,'Parameter to be displayed in the console window');

      // second parameter is an integer
      jv.i:=2333;
      x[1]:=jv.l;
      JavaEnv^.CallVoidMethodV(JavaEnv,JavaObjectID,stuntMethod,x); // (Ljava/lang/String;I)V

      ji:=JavaEnv^.CallIntMethod(JavaEnv,JavaObjectID,GetMethod);
      memo1.Lines.Add('After stunt '+inttostr(ji));

    end
    else
    begin
      memo1.Lines.Add('?Could not derive ILOCALOBJECT');
      exit;
    end;
  end
  else
    memo1.Lines.Add('?'+javaClassname+' not found');
end;

function TForm2.CheckForCustomJavaJarFile: Boolean;
var
  sPath : String;
begin
  Result := False;
  sPath := TPath.GetDocumentsPath + PathDelim + CUSTOMJARFILE;
  if (TFile.Exists(sPath)) then
  begin
    Memo1.Lines.Add(sPath);
    Result := True;
  end
  else
    Memo1.Lines.Text := TPath.GetDocumentsPath + PathDelim;
end;

constructor TForm2.Create(AOwner: TComponent);
begin
  inherited;
end;

destructor TForm2.Destroy;
begin
  inherited;
end;

procedure TForm2.DoSetups;
begin
  if (CheckForCustomJavaJarFile) then
  begin
    LoadCustomJavaClass;
  end;
end;

procedure TForm2.FormCreate(Sender: TObject);
begin
  DoSetups;
end;

procedure TForm2.LoadCustomJavaClass;
begin

end;

end.
